/*
    This is a romfs file system driver for Windows NT and Windows 2000,
    pre-release 2000-07-11.
    Copyright (C) 2000 Bo Branten
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "ntifs.h"
#include "fsd.h"

PVOID
FsdAllocatePool (
    IN POOL_TYPE    PoolType,
    IN ULONG        NumberOfBytes
    )
{
    PVOID P;

#if DBG

    NumberOfBytes += sizeof(FSD_ALLOC_HEADER);

#endif // DBG

    P = ExAllocatePool(PoolType, NumberOfBytes);

#if DBG

    if (P)
    {
        RtlFillMemory(P, NumberOfBytes, 'A');

        ((PFSD_ALLOC_HEADER)P)->Identifier.Type = 'AAAA';
        ((PFSD_ALLOC_HEADER)P)->Identifier.Size = NumberOfBytes;

        return (PVOID)((PUCHAR)P + sizeof(FSD_ALLOC_HEADER));
    }
    else
    {
        KdPrint((DRIVER_NAME ": FsdAllocatePool: ExAllocatePool failed\n"));

        return NULL;
    }

#else // !DBG

    return P;

#endif // !DBG

}

VOID
FsdFreePool (
    IN PVOID P,
    IN ULONG NumberOfBytes
    )
{
    ASSERT(P != NULL);

#if DBG

    NumberOfBytes += sizeof(FSD_ALLOC_HEADER);

    P = (PVOID)((PUCHAR)P - sizeof(FSD_ALLOC_HEADER));

    ASSERT(((PFSD_ALLOC_HEADER)P)->Identifier.Type == 'AAAA');
    ASSERT(((PFSD_ALLOC_HEADER)P)->Identifier.Size == NumberOfBytes);

    RtlFillMemory(P, NumberOfBytes, 'F');

#endif // !DBG

    ExFreePool(P);
}

PFSD_IRP_CONTEXT
FsdAllocateIrpContext (
    IN PDEVICE_OBJECT   DeviceObject,
    IN PIRP             Irp
    )
{
    PIO_STACK_LOCATION  IoStackLocation;
    PFSD_IRP_CONTEXT    IrpContext;

    ASSERT(DeviceObject != NULL);
    ASSERT(Irp != NULL);

    IoStackLocation = IoGetCurrentIrpStackLocation(Irp);

    //
    // Allocate an IrpContext
    //

    IrpContext = FsdAllocatePool(NonPagedPool, sizeof(FSD_IRP_CONTEXT));

    if (!IrpContext)
    {
        return NULL;
    }

    //
    // Initialize the IrpContext with information about the request
    //

    IrpContext->Identifier.Type = ICX;
    IrpContext->Identifier.Size = sizeof(FSD_IRP_CONTEXT);

    IrpContext->Irp = Irp;

    IrpContext->MajorFunction = IoStackLocation->MajorFunction;
    IrpContext->MinorFunction = IoStackLocation->MinorFunction;

    IrpContext->DeviceObject = DeviceObject;

    IrpContext->FileObject = IoStackLocation->FileObject;

    if (IrpContext->MajorFunction == IRP_MJ_FILE_SYSTEM_CONTROL ||
        IrpContext->MajorFunction == IRP_MJ_DEVICE_CONTROL ||
        IrpContext->MajorFunction == IRP_MJ_SHUTDOWN)
    {
        IrpContext->IsSynchronous = TRUE;
    }
    else if (IrpContext->MajorFunction == IRP_MJ_CLEANUP)
    {
        IrpContext->IsSynchronous = FALSE;
    }
    else
    {
        IrpContext->IsSynchronous = IoIsOperationSynchronous(Irp);
    }

    IrpContext->IsTopLevel = (IoGetTopLevelIrp() == Irp);

    IrpContext->ExceptionInProgress = FALSE;

    return IrpContext;
}

VOID
FsdFreeIrpContext (
    IN PFSD_IRP_CONTEXT IrpContext
    )
{
    ASSERT(IrpContext != NULL);

    ASSERT((IrpContext->Identifier.Type == ICX) &&
           (IrpContext->Identifier.Size == sizeof(FSD_IRP_CONTEXT)));

    FsdFreePool(IrpContext, sizeof(FSD_IRP_CONTEXT));
}
