/*
    This is a romfs file system driver for Windows NT and Windows 2000,
    pre-release 2000-07-11.
    Copyright (C) 2000 Bo Branten
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "ntifs.h"
#include "fsd.h"
#include "rom_fs.h"
#include "border.h"

NTSTATUS
FsdFileSystemControl (
    IN PFSD_IRP_CONTEXT IrpContext
    )
{
    NTSTATUS Status;

    ASSERT(IrpContext);

    ASSERT((IrpContext->Identifier.Type == ICX) &&
           (IrpContext->Identifier.Size == sizeof(FSD_IRP_CONTEXT)));

    switch (IrpContext->MinorFunction)
    {
    /*
    case IRP_MN_USER_FS_REQUEST:
        Status = FsdUserFsRequest(IrpContext);
        break;
    */

    case IRP_MN_MOUNT_VOLUME:
        Status = FsdMountVolume(IrpContext);
        break;

    /*
    case IRP_MN_VERIFY_VOLUME:
        Status = FsdVerifyVolume(IrpContext);
        break;
    */

    default:
        KdPrint((DRIVER_NAME ": FsdFileSystemControl: "
            "Unknown minor function 0x%x\n", IrpContext->MinorFunction));
        Status = STATUS_NOT_IMPLEMENTED;
        IrpContext->Irp->IoStatus.Status = Status;
        FsdCompleteRequest(IrpContext->Irp, IO_NO_INCREMENT);
        FsdFreeIrpContext(IrpContext);
    }

    return Status;
}

/*
NTSTATUS
FsdUserFsRequest (
    IN PFSD_IRP_CONTEXT IrpContext
    )
{
}
*/

NTSTATUS
FsdMountVolume (
    IN PFSD_IRP_CONTEXT IrpContext
    )
{
    PIRP                Irp;
    PIO_STACK_LOCATION  io_stack;
    NTSTATUS                Status = STATUS_UNSUCCESSFUL;
    PDEVICE_OBJECT      device_object = NULL;
    PUCHAR          buffer;
    LARGE_INTEGER   offset;
    ULONG						size;
    PFSD_VCB            Vcb;
    
    __try
    {
        KdPrint((DRIVER_NAME ": FileSystemControl: IRP_MN_MOUNT_VOLUME\n"));

        Irp = IrpContext->Irp;
        
        io_stack = IoGetCurrentIrpStackLocation(Irp);

        buffer = (PUCHAR) ExAllocatePool(NonPagedPoolCacheAligned, 512);

        if (!buffer)
        {
            Status = STATUS_INSUFFICIENT_RESOURCES;
            __leave;
        }
        
        offset.QuadPart = 0;
        
        Status = FsdReadBlockDevice(
            io_stack->Parameters.MountVolume.DeviceObject,
            &offset,
            512,
            buffer
            );

        if (!NT_SUCCESS(Status))
        {
            __leave;
        }

        if (!(buffer[0] == '-'  &&  buffer[1] == 'r'))
        {
            Status = STATUS_UNRECOGNIZED_VOLUME;
            __leave;
        }

        Status = IoCreateDevice(
            IrpContext->DeviceObject->DriverObject,
            sizeof(FSD_VCB),
            NULL,
            FILE_DEVICE_DISK_FILE_SYSTEM,
            0,
            FALSE,
            &device_object
            );
        
        if (!NT_SUCCESS(Status))
        {
            __leave;
        }
        
        Vcb = (PFSD_VCB)device_object->DeviceExtension;
        
        device_object->Flags &= ~DO_DEVICE_INITIALIZING;
        
        device_object->StackSize = (io_stack->Parameters.MountVolume.DeviceObject)->StackSize;
        
        (io_stack->Parameters.MountVolume.Vpb)->DeviceObject = device_object;
        
        ExInitializeResourceLite(&((PFSD_VCB)device_object->DeviceExtension)->MainResource);
        
        ((PFSD_VCB)device_object->DeviceExtension)->TargetDeviceObject =
            io_stack->Parameters.MountVolume.DeviceObject;
        
        ((PFSD_VCB)device_object->DeviceExtension)->Identifier.Type = VCB;
        ((PFSD_VCB)device_object->DeviceExtension)->Identifier.Size = sizeof(FSD_VCB);
        
        
        ((PFSD_VCB)device_object->DeviceExtension)->romfs_super_block = (struct romfs_super_block*)buffer;
        ((PFSD_VCB)device_object->DeviceExtension)->MaximumVolumeLabelLength = 16;
        
        size = sizeof(DISK_GEOMETRY);
        
        Status = FsdBlockDeviceIoControl(
            io_stack->Parameters.MountVolume.DeviceObject,
            IOCTL_DISK_GET_DRIVE_GEOMETRY,
            NULL,
            0,
            &Vcb->DiskGeometry,
            &size
            );
        
        size = sizeof(PARTITION_INFORMATION);
        
        Status = FsdBlockDeviceIoControl(
            io_stack->Parameters.MountVolume.DeviceObject,
            IOCTL_DISK_GET_PARTITION_INFO,
            NULL,
            0,
            &Vcb->PartitionInformation,
            &size
            );
    }
    __finally
    {
        /*if (VcbResourceAcquired)
        {
            ExReleaseResourceForThreadLite(
                &Vcb->MainResource,
                ExGetCurrentResourceThread()
                );
        }*/
        
        /*if (device_object)
        {
            IoDeleteDevice(device_object);
        }*/
        
        if (!IrpContext->ExceptionInProgress)
        {
            if (!NT_SUCCESS(Status))
            {
                Status = STATUS_UNRECOGNIZED_VOLUME;
            }
            
            IrpContext->Irp->IoStatus.Status = Status;
            
            /*FsdCompleteRequest(
                IrpContext->Irp,
                (CCHAR)
                (NT_SUCCESS(Status) ? IO_DISK_INCREMENT : IO_NO_INCREMENT)
                );*/
            
            FsdCompleteRequest(
                IrpContext->Irp,
                (CCHAR)
                IO_NO_INCREMENT
                );

            FsdFreeIrpContext(IrpContext);
        }
    }
    
    return Status;
}

#if 0
NTSTATUS
FsdMountVolume (
    IN PFSD_IRP_CONTEXT IrpContext
    )
{
    PDEVICE_OBJECT      DeviceObject;
    PIRP                Irp;
    PIO_STACK_LOCATION  io_stack;
    NTSTATUS            status;
    PDEVICE_OBJECT      device_object;
    PUCHAR          buffer;
    LARGE_INTEGER   offset;
	ULONG						size;
    PFSD_VCB            Vcb;

    buffer = (PUCHAR) ExAllocatePool(NonPagedPoolCacheAligned, 512);

    if (!buffer)
    {
        return STATUS_INSUFFICIENT_RESOURCES;
    }

    offset.QuadPart = 0;

    DeviceObject = IrpContext->DeviceObject;
    Irp = IrpContext->Irp;

    io_stack = IoGetCurrentIrpStackLocation(Irp);

    ASSERT(io_stack != NULL);

        KdPrint((DRIVER_NAME ": FileSystemControl: IRP_MN_MOUNT_VOLUME\n"));

        status = IoCreateDevice(
            DeviceObject->DriverObject,
            sizeof(FSD_VCB),
            NULL,
            FILE_DEVICE_DISK_FILE_SYSTEM,
            0,
            FALSE,
            &device_object
            );

        if (!NT_SUCCESS(status))
        {
            Irp->IoStatus.Status = status;
            Irp->IoStatus.Information = 0;
            FsdCompleteRequest(Irp, IO_NO_INCREMENT);

            return status;
        }

        //VolumeDeviceObject = device_object;
        Vcb = (PFSD_VCB)device_object->DeviceExtension;

        device_object->Flags &= ~DO_DEVICE_INITIALIZING;

        device_object->StackSize = (io_stack->Parameters.MountVolume.DeviceObject)->StackSize;

        (io_stack->Parameters.MountVolume.Vpb)->DeviceObject = device_object;

        ExInitializeResourceLite(&((PFSD_VCB)device_object->DeviceExtension)->MainResource);

        ((PFSD_VCB)device_object->DeviceExtension)->TargetDeviceObject =
            io_stack->Parameters.MountVolume.DeviceObject;

        ((PFSD_VCB)device_object->DeviceExtension)->Identifier.Type = VCB;
        ((PFSD_VCB)device_object->DeviceExtension)->Identifier.Size = sizeof(FSD_VCB);

        status = FsdReadBlockDevice(
            io_stack->Parameters.MountVolume.DeviceObject,
            &offset,
            512,
            buffer
            );

        ((PFSD_VCB)device_object->DeviceExtension)->romfs_super_block = (struct romfs_super_block*)buffer;
        ((PFSD_VCB)device_object->DeviceExtension)->MaximumVolumeLabelLength = 16;

        if (buffer[0] == '-'  &&  buffer[1] == 'r')
        {
            status = STATUS_SUCCESS;
        }
        else
        {
            status = STATUS_UNRECOGNIZED_VOLUME;
        }

	size = sizeof(DISK_GEOMETRY);

	FsdBlockDeviceIoControl(
		io_stack->Parameters.MountVolume.DeviceObject,
		IOCTL_DISK_GET_DRIVE_GEOMETRY,
		NULL,
		0,
        &Vcb->DiskGeometry,
		&size
		);

	size = sizeof(PARTITION_INFORMATION);

	FsdBlockDeviceIoControl(
		io_stack->Parameters.MountVolume.DeviceObject,
		IOCTL_DISK_GET_PARTITION_INFO,
		NULL,
		0,
        &Vcb->PartitionInformation,
		&size
		);

	/*FsdBlockDeviceIoControl(
		device_object,
		IOCTL_DISK_GET_PARTITION_INFO,
		NULL,
		0,
        &Vcb->PartitionInformation,
		&size
		);*/

        Irp->IoStatus.Status = status;
        Irp->IoStatus.Information = 0;
        FsdCompleteRequest(Irp, IO_NO_INCREMENT);
        FsdFreeIrpContext(IrpContext);

        return status;
}
#endif

/*
NTSTATUS
FsdVerifyVolume (
    IN PFSD_IRP_CONTEXT IrpContext
    )
{
}
*/
