/*
    This is a romfs file system driver for Windows NT and Windows 2000,
    pre-release 2000-07-11.
    Copyright (C) 2000 Bo Branten
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "ntifs.h"
#include "fsd.h"

//
// The FSDs global data
//
FSD_GLOBAL_DATA FsdGlobalData;

#pragma code_seg("init")

NTSTATUS
DriverEntry (
    IN PDRIVER_OBJECT   DriverObject,
    IN PUNICODE_STRING  RegistryPath
    )
{
    PFAST_IO_DISPATCH           FastIoDispatch;
    PCACHE_MANAGER_CALLBACKS    CacheManagerCallbacks;
    UNICODE_STRING              DeviceName;
    NTSTATUS                    Status;

    KdPrint((DRIVER_NAME ": DriverEntry\n"));

    //
    // Initialize the global data
    //

    RtlZeroMemory(&FsdGlobalData, sizeof(FSD_GLOBAL_DATA));

    FsdGlobalData.Identifier.Type = FGD;
    FsdGlobalData.Identifier.Size = sizeof(FSD_GLOBAL_DATA);

    ExInitializeResourceLite(&FsdGlobalData.Resource);

    FsdGlobalData.DriverObject = DriverObject;

    InitializeListHead(&FsdGlobalData.VcbList);

    //
    // Initialize the dispatch entry points
    //

    DriverObject->MajorFunction[IRP_MJ_CREATE] = FsdBuildRequest;
    DriverObject->MajorFunction[IRP_MJ_CLOSE] = FsdBuildRequest;
    DriverObject->MajorFunction[IRP_MJ_READ] = FsdBuildRequest;
    DriverObject->MajorFunction[IRP_MJ_QUERY_INFORMATION] = FsdBuildRequest;
    DriverObject->MajorFunction[IRP_MJ_SET_INFORMATION] = FsdBuildRequest;
    //DriverObject->MajorFunction[IRP_MJ_QUERY_EA] = 
    DriverObject->MajorFunction[IRP_MJ_FLUSH_BUFFERS] = FsdBuildRequest;
    DriverObject->MajorFunction[IRP_MJ_QUERY_VOLUME_INFORMATION] =
        FsdBuildRequest;
    DriverObject->MajorFunction[IRP_MJ_DIRECTORY_CONTROL] = FsdBuildRequest;
    DriverObject->MajorFunction[IRP_MJ_FILE_SYSTEM_CONTROL] = FsdBuildRequest;
    DriverObject->MajorFunction[IRP_MJ_DEVICE_CONTROL] = FsdBuildRequest;
    DriverObject->MajorFunction[IRP_MJ_SHUTDOWN] = FsdBuildRequest;
    //DriverObject->MajorFunction[IRP_MJ_LOCK_CONTROL] = 
    DriverObject->MajorFunction[IRP_MJ_CLEANUP] = FsdBuildRequest;
    //DriverObject->MajorFunction[IRP_MJ_QUERY_SECURITY] = 
    //DriverObject->MajorFunction[IRP_MJ_QUERY_QUOTA] = 

#ifndef FSD_RO

    //DriverObject->MajorFunction[IRP_MJ_WRITE] = 
    //DriverObject->MajorFunction[IRP_MJ_SET_EA] = 
    DriverObject->MajorFunction[IRP_MJ_SET_VOLUME_INFORMATION] =
        FsdBuildRequest;
    //DriverObject->MajorFunction[IRP_MJ_SET_SECURITY] = 
    //DriverObject->MajorFunction[IRP_MJ_SET_QUOTA] = 

#endif // !FSD_RO

    //
    // Initialize the fast I/O entry points
    //

    FastIoDispatch = &FsdGlobalData.FastIoDispatch;

    FastIoDispatch->SizeOfFastIoDispatch = sizeof(FAST_IO_DISPATCH);
    FastIoDispatch->FastIoCheckIfPossible = FsdFastIoCheckIfPossible;
    FastIoDispatch->FastIoRead = FsdFastIoRead;
    FastIoDispatch->FastIoWrite = FsdFastIoWrite;
    FastIoDispatch->FastIoQueryBasicInfo = FsdFastIoQueryBasicInfo;
    FastIoDispatch->FastIoQueryStandardInfo = FsdFastIoQueryStandardInfo;
    FastIoDispatch->FastIoLock = FsdFastIoLock;
    FastIoDispatch->FastIoUnlockSingle = FsdFastIoUnlockSingle;
    FastIoDispatch->FastIoUnlockAll = FsdFastIoUnlockAll;
    FastIoDispatch->FastIoUnlockAllByKey = FsdFastIoUnlockAllByKey;
    FastIoDispatch->FastIoDeviceControl = FsdFastIoDeviceControl;
    FastIoDispatch->AcquireFileForNtCreateSection =
        FsdFastIoAcquireFileForNtCreateSection;
    FastIoDispatch->ReleaseFileForNtCreateSection =
        FsdFastIoReleaseFileForNtCreateSection;
    FastIoDispatch->FastIoDetachDevice = FsdFastIoDetachDevice;
    FastIoDispatch->FastIoQueryNetworkOpenInfo = FsdFastIoQueryNetworkOpenInfo;
    FastIoDispatch->AcquireForModWrite = FsdFastIoAcquireForModWrite;
    FastIoDispatch->MdlRead = FsdFastIoMdlRead;
    FastIoDispatch->MdlReadComplete = FsdFastIoMdlReadComplete;
    FastIoDispatch->PrepareMdlWrite = FsdFastIoPrepareMdlWrite;
    FastIoDispatch->MdlWriteComplete = FsdFastIoMdlWriteComplete;
    //FastIoDispatch->FastIoReadCompressed = 
    //FastIoDispatch->FastIoWriteCompressed = 
    //FastIoDispatch->MdlReadCompleteCompressed = 
    //FastIoDispatch->MdlWriteCompleteCompressed = 
    FastIoDispatch->FastIoQueryOpen = FsdFastIoQueryOpen;
    FastIoDispatch->ReleaseForModWrite = FsdFastIoReleaseForModWrite;
    FastIoDispatch->AcquireForCcFlush = FsdFastIoAcquireForCcFlush;
    FastIoDispatch->ReleaseForCcFlush = FsdFastIoReleaseForCcFlush;

    DriverObject->FastIoDispatch = FastIoDispatch;

    //
    // Initialize the Cache Manager callbacks
    //

    CacheManagerCallbacks = &FsdGlobalData.CacheManagerCallbacks;

    CacheManagerCallbacks->AcquireForLazyWrite = FsdAcquireForLazyWrite;
    CacheManagerCallbacks->ReleaseFromLazyWrite = FsdReleaseFromLazyWrite;
    CacheManagerCallbacks->AcquireForReadAhead = FsdAcquireForReadAhead;
    CacheManagerCallbacks->ReleaseFromReadAhead = FsdReleaseFromReadAhead;

    //
    // Create the main device object
    //

    RtlInitUnicodeString(&DeviceName, DEVICE_NAME);

    Status = IoCreateDevice(
        DriverObject,
        0,
        &DeviceName,
        FILE_DEVICE_DISK_FILE_SYSTEM,
        0,
        FALSE,
        &FsdGlobalData.DeviceObject
        );

    if (NT_SUCCESS(Status))
    {
        IoRegisterFileSystem(FsdGlobalData.DeviceObject);
    }
    else
    {
        KdPrint((DRIVER_NAME ": DriverEntry: IoCreateDevice failed\n"));

        ExDeleteResourceLite(&FsdGlobalData.Resource);
    }

    return Status;
}

#pragma code_seg()
